-- Contacts prefix: co_ systemid: 1
-- Tables for the Contacts Module
-- Copyright (c) 2000 The FreeMoney Project
-- This work is released under the GPL
-- $Id: create_tables_contact.sql,v 1.34 2001/10/14 07:31:24 kaare Exp $

-- Sequences
DROP SEQUENCE co_countrysequence;
DROP SEQUENCE co_contactsequence;
DROP SEQUENCE co_phonesequence;
DROP SEQUENCE co_urlsequence;
DROP SEQUENCE co_addresssequence;
DROP SEQUENCE co_notesequence;
DROP SEQUENCE co_selectsequence;
DROP SEQUENCE co_bsectsequence;

CREATE SEQUENCE co_countrysequence INCREMENT 1;
CREATE SEQUENCE co_contactsequence INCREMENT 1;
CREATE SEQUENCE co_phonesequence   INCREMENT 1;
CREATE SEQUENCE co_urlsequence     INCREMENT 1;
CREATE SEQUENCE co_addresssequence INCREMENT 1;
CREATE SEQUENCE co_notesequence    INCREMENT 1;
CREATE SEQUENCE co_selectsequence  INCREMENT 1;
CREATE SEQUENCE co_bsectsequence INCREMENT 1;

--Tables
DROP TABLE co_contact;
DROP TABLE co_postalcodes;
DROP TABLE co_company;
DROP TABLE co_person;
DROP TABLE co_telephone;
DROP TABLE co_url;
DROP TABLE co_address;
DROP TABLE co_customer;
DROP TABLE co_businesssector;
DROP TABLE co_creditterms;
DROP TABLE co_deliveryterms;
DROP TABLE co_paymentterms;
DROP TABLE co_freightcode;
DROP TABLE co_customerresponsibility;
DROP TABLE co_contactnotes;
DROP TABLE co_selection;
DROP TABLE co_contactselect;

-- First the contacts table

CREATE TABLE co_contact (
       contact_id           integer NOT NULL DEFAULT nextval('co_contactsequence'),
       contact_nr           decimal,
       type                 integer,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (contact_id ),
 FOREIGN KEY (type)         REFERENCES sy_typetext
);

COMMENT ON TABLE co_contact IS 'Central table for all contact information
Every contact, independent of type (customer, employee, lessor...) will be here';
COMMENT ON COLUMN co_contact.contact_id  IS 'Sequence number';
COMMENT ON COLUMN co_contact.contact_nr IS '??';
COMMENT ON COLUMN co_contact.type IS 'Type. E.g. 1 - Customer 2 - Owner company 3 - Employee';
COMMENT ON COLUMN co_contact.updated IS 'Timestamp for last update of this row';

-- Then address stuff
CREATE TABLE co_postalcodes (
       country_id           integer NOT NULL,
       postalcode           varchar(20) NOT NULL,
       name                 varchar(128),
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (country_id , postalcode),
 FOREIGN KEY (country_id ) REFERENCES co_country
);

COMMENT ON TABLE co_postalcodes IS 'Postal (zip) codes';
COMMENT ON COLUMN co_postalcodes.country_id  IS '';
COMMENT ON COLUMN co_postalcodes.postalcode IS '';
COMMENT ON COLUMN co_postalcodes.name IS '';
COMMENT ON COLUMN co_postalcodes.updated IS 'Timestamp for last update of this row';

CREATE TABLE co_address (
       address_id           integer NOT NULL DEFAULT nextval('co_addresssequence'),
       country_id           integer NOT NULL,
       contact_id           integer NOT NULL,
       type                 integer,
       address1             varchar(128),
       address2             varchar(128),
       address3             varchar(128),
       city                 varchar(20),
       state                varchar(20),
       postalcode           varchar(20),
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (address_id ),
 FOREIGN KEY (contact_id )  REFERENCES co_contact
  ON DELETE CASCADE
  ON UPDATE CASCADE,
 FOREIGN KEY (type)         REFERENCES sy_typetext,
 FOREIGN KEY (country_id , postalcode) REFERENCES co_postalcodes,
 FOREIGN KEY (country_id)   REFERENCES co_country
);

COMMENT ON TABLE co_address IS 'All addresses are placed here';
COMMENT ON COLUMN co_address.address_id  IS 'Sequence number';
COMMENT ON COLUMN co_address.country_id  IS 'In which is the address?';
COMMENT ON COLUMN co_address.contact_id  IS 'The person or company at that address';
COMMENT ON COLUMN co_address.type IS 'Type. E.g. 1 - Main address 2 - Delivery address 3 - Payment address';
COMMENT ON COLUMN co_address.address1 IS 'First address line';
COMMENT ON COLUMN co_address.address2 IS 'Second  address line';
COMMENT ON COLUMN co_address.address3 IS 'Third  address line';
COMMENT ON COLUMN co_address.city IS 'City';
COMMENT ON COLUMN co_address.state IS 'State';
COMMENT ON COLUMN co_address.postalcode IS 'Postal code';
COMMENT ON COLUMN co_address.updated IS 'Timestamp for last update of this row';

CREATE TABLE co_url (
       url_id               integer NOT NULL DEFAULT nextval('co_urlsequence'),
       contact_id           integer,
       type                 integer,
       url                  varchar(128),
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (url_id ),
 FOREIGN KEY (contact_id )  REFERENCES co_contact
  ON DELETE CASCADE
  ON UPDATE CASCADE,
 FOREIGN KEY (type)         REFERENCES sy_typetext
);

COMMENT ON TABLE co_url IS 'Internet addresses';
COMMENT ON COLUMN co_url.url_id  IS 'Sequence number';
COMMENT ON COLUMN co_url.contact_id  IS 'Reference to contact';
COMMENT ON COLUMN co_url.type IS 'Type. E.g. 1 - Web site 2 - email 3 - ftp';
COMMENT ON COLUMN co_url.url IS 'The Internet address';
COMMENT ON COLUMN co_url.updated IS 'Timestamp for last update of this row';

CREATE TABLE co_telephone (
       phone_id             integer NOT NULL DEFAULT nextval('co_phonesequence'),
       contact_id           integer,
       type                 integer,
       phonenr             varchar(128),
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (phone_id ),
 FOREIGN KEY (contact_id )  REFERENCES co_contact
  ON DELETE CASCADE
  ON UPDATE CASCADE,
 FOREIGN KEY (type)         REFERENCES sy_typetext
);

COMMENT ON TABLE co_telephone IS 'Telephone numbers';
COMMENT ON COLUMN co_telephone.phone_id  IS 'Sequence number';
COMMENT ON COLUMN co_telephone.contact_id  IS 'Reference to contact';
COMMENT ON COLUMN co_telephone.type IS 'Type. E.g. 1 - main line 2 - cell phone';
COMMENT ON COLUMN co_telephone.phonenr IS 'The number';
COMMENT ON COLUMN co_telephone.updated IS 'Timestamp for last update of this row';

CREATE TABLE co_company (
       contact_id           integer NOT NULL,
       name                 varchar(128),
       regnr                varchar(20),
       attention            varchar(128),
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (contact_id ),
 FOREIGN KEY (contact_id )  REFERENCES co_contact
  ON DELETE CASCADE
  ON UPDATE CASCADE
);

COMMENT ON TABLE co_company IS 'Company information';
COMMENT ON COLUMN co_company.contact_id  IS 'Reference to contact';
COMMENT ON COLUMN co_company.name IS 'Company name';
COMMENT ON COLUMN co_company.regnr IS 'Officiel registration number';
COMMENT ON COLUMN co_company.attention IS 'Contact person';
COMMENT ON COLUMN co_company.updated IS 'Timestamp for last update of this row';

CREATE TABLE co_person (
       contact_id           integer NOT NULL,
       firstname            varchar(128),
       lastname             varchar(20),
       personid             varchar(20),
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (contact_id ),
 FOREIGN KEY (contact_id )  REFERENCES co_contact
  ON DELETE CASCADE
  ON UPDATE CASCADE
);

COMMENT ON TABLE co_person IS 'Information about a person';
COMMENT ON COLUMN co_person.contact_id  IS 'Reference to contact';
COMMENT ON COLUMN co_person.firstname IS 'First name';
COMMENT ON COLUMN co_person.lastname IS 'Last name';
COMMENT ON COLUMN co_person.personid IS 'Personal ID. E.g. social security number';
COMMENT ON COLUMN co_company.updated IS 'Timestamp for last update of this row';

-- Customer tables
CREATE TABLE co_businesssector (
       bsect_id             integer NOT NULL DEFAULT nextval('co_bsectsequence'),
       name                 varchar(128) NOT NULL,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (bsect_id )
);

COMMENT ON TABLE co_businesssector IS 'Some kind of business information. Idea not developed.';
COMMENT ON COLUMN co_businesssector.bsect_id  IS 'Sequence number';
COMMENT ON COLUMN co_businesssector.name IS 'Business sector name';
COMMENT ON COLUMN co_businesssector.updated IS 'Timestamp for last update of this row';

CREATE TABLE co_creditterms (
       creditcode          integer NOT NULL,
       name                 varchar(128),
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (creditcode)
);

COMMENT ON TABLE co_creditterms IS 'Credit terms';
COMMENT ON COLUMN co_creditterms.creditcode IS 'Credit code. Unique ID';
COMMENT ON COLUMN co_creditterms.name IS 'Name';
COMMENT ON COLUMN co_creditterms.updated IS 'Timestamp for last update of this row';

CREATE TABLE co_deliveryterms (
       deliverycode         integer NOT NULL,
       name                 varchar(128),
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (deliverycode)
);

COMMENT ON TABLE co_deliveryterms IS 'Delivery terms';
COMMENT ON COLUMN co_deliveryterms.deliverycode IS 'Delivery code. Unique ID';
COMMENT ON COLUMN co_deliveryterms.name IS 'Name';
COMMENT ON COLUMN co_deliveryterms.updated IS 'Timestamp for last update of this row';

CREATE TABLE co_paymentterms (
       paycode              integer NOT NULL,
       name                 varchar(128),
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (paycode)
);

COMMENT ON TABLE co_paymentterms IS 'Payment terms';
COMMENT ON COLUMN co_paymentterms.paycode IS 'Payment code. Unique ID';
COMMENT ON COLUMN co_paymentterms.name IS 'Name';
COMMENT ON COLUMN co_paymentterms.updated IS 'Timestamp for last update of this row';

CREATE TABLE co_freightcode (
       freightcode          integer NOT NULL,
       name                 varchar(128) NOT NULL,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (freightcode)
);

COMMENT ON TABLE co_freightcode IS 'Freight code';
COMMENT ON COLUMN co_freightcode.freightcode IS 'Freight code. Unique ID';
COMMENT ON COLUMN co_freightcode.name IS 'Name';
COMMENT ON COLUMN co_freightcode.updated IS 'Timestamp for last update of this row';

CREATE TABLE co_customer (
       contact_id           integer NOT NULL,
       type                 integer,
       customernr           varchar(20),
       custgrp_id           integer,
       bsect_id             integer,
       credit_code          integer,
       deliverycode         integer,
       paycode              integer,
       freightcode          integer,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (contact_id),
 FOREIGN KEY (bsect_id )    REFERENCES co_businesssector,
 FOREIGN KEY (credit_code)  REFERENCES co_creditterms,
 FOREIGN KEY (deliverycode) REFERENCES co_deliveryterms,
 FOREIGN KEY (paycode)      REFERENCES co_paymentterms,
 FOREIGN KEY (freightcode)  REFERENCES co_freightcode,
 FOREIGN KEY (custgrp_id )  REFERENCES co_custgrp,
 FOREIGN KEY (type)         REFERENCES sy_typetext,
 --FOREIGN KEY (customernr)   REFERENCES userdb (username)
--  ON DELETE SET NULL
--  ON UPDATE CASCADE,
 FOREIGN KEY (contact_id )  REFERENCES co_contact
  ON DELETE CASCADE
  ON UPDATE CASCADE
);

COMMENT ON TABLE co_customer IS 'Customers';
COMMENT ON COLUMN co_customer.contact_id  IS 'Reference to contact';
COMMENT ON COLUMN co_customer.type IS 'Customer type. E.g. ??';
COMMENT ON COLUMN co_customer.customernr IS 'Customer number';
COMMENT ON COLUMN co_customer.custgrp_id  IS 'Reference to customer group';
COMMENT ON COLUMN co_customer.bsect_id  IS 'Reference to business sector. Currently not used';
COMMENT ON COLUMN co_customer.credit_code IS 'Credit code';
COMMENT ON COLUMN co_customer.deliverycode IS 'Delivery code';
COMMENT ON COLUMN co_customer.paycode IS 'Payment code';
COMMENT ON COLUMN co_customer.freightcode IS 'Freight code';
COMMENT ON COLUMN co_customer.updated IS 'Timestamp for last update of this row';

CREATE TABLE co_customerresponsibility (
       contact_id           integer NOT NULL,
       name                 varchar(128),
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (contact_id ),
 FOREIGN KEY (contact_id )  REFERENCES co_customer
);

COMMENT ON TABLE co_customerresponsibility IS 'Forgot what this table is for...';
COMMENT ON COLUMN co_customerresponsibility.contact_id  IS 'Reference to contact';
COMMENT ON COLUMN co_customerresponsibility.name IS 'Name';
COMMENT ON COLUMN co_customerresponsibility.updated IS 'Timestamp for last update of this row';

-- Notes
CREATE TABLE co_contactnotes (
       note_id              integer NOT NULL DEFAULT nextval('co_notesequence'),
       contact_id           integer,
       Description          varchar(128),
       enddate              timestamp,
       end_event            varchar(128),
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (note_id ),
 FOREIGN KEY (contact_id )  REFERENCES co_contact
  ON DELETE CASCADE
  ON UPDATE CASCADE
);

COMMENT ON TABLE co_contactnotes IS 'Contact notes.
The idea is to have an unlimited number of notes per customer, person or company';
COMMENT ON COLUMN co_contactnotes.note_id  IS 'Sequence number';
COMMENT ON COLUMN co_contactnotes.contact_id  IS 'Which contact?';
COMMENT ON COLUMN co_contactnotes.description IS 'The note';
COMMENT ON COLUMN co_contactnotes.enddate IS 'Maybe it will expire someday';
COMMENT ON COLUMN co_contactnotes.end_event IS 'Or maybe it will end because something happens';
COMMENT ON COLUMN co_contactnotes.updated IS 'Timestamp for last update of this row';

-- Selection
CREATE TABLE co_selection (
       select_id            integer NOT NULL DEFAULT nextval('co_selectsequence'),
       name                 varchar(128),
       action               varchar(128),
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (select_id )
);

COMMENT ON TABLE co_selection IS 'Contact selection.
The idea is to be able to select customers, persons or whatever either by choosing them individually, or by some rule
This could be used to find all customers with overdue payments and send them a reminder,
find potential customers for a product and much more';
COMMENT ON COLUMN co_selection.select_id  IS 'Sequence number';
COMMENT ON COLUMN co_selection.name IS 'Name';
COMMENT ON COLUMN co_selection.action IS 'Some kind of selection rule, Not yet defined how this works';
COMMENT ON COLUMN co_selection.updated IS 'Timestamp for last update of this row';

CREATE TABLE co_contactselect (
       select_id            integer NOT NULL,
       contact_id           integer NOT NULL,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (select_id , contact_id ),
 FOREIGN KEY (select_id )   REFERENCES co_selection,
 FOREIGN KEY (contact_id )  REFERENCES co_contact
  ON DELETE CASCADE
  ON UPDATE CASCADE
);

COMMENT ON TABLE co_contactselect IS 'Table for holding the selected contacts. 
The idea is to store selected customers until they''re actively released by the user';
COMMENT ON COLUMN co_contactselect.select_id  IS 'Reference to selection';
COMMENT ON COLUMN co_contactselect.contact_id  IS 'Reference to contact';
COMMENT ON COLUMN co_contactselect.updated IS 'Timestamp for last update of this row';

-- Views
DROP VIEW co_custaddr_v;

CREATE VIEW co_custaddr_v AS
 SELECT cu.contact_id , cu.customernr, cu.type AS custtype, cu.custgrp_id,
   (SELECT lastname||', '||firstname FROM co_person p
     WHERE p.contact_id  = cu.contact_id
    UNION
    SELECT name FROM co_company c
     WHERE c.contact_id  = cu.contact_id) AS name,
   ad.address1, ad.address2, ad.address3, ad.country_id,
   po.postalcode, po.name as city,
   te.phonenr, te.type as phonetype,
   u.url, u.type as urltype
  FROM co_customer cu
  LEFT JOIN co_telephone te ON (cu.contact_id  = te.contact_id AND te.type = 90001)
  LEFT JOIN co_url u ON (cu.contact_id  = u.contact_id AND u.type = 80001),
   co_address ad, co_postalcodes po
  WHERE cu.contact_id  = ad.contact_id
  AND ad.country_id  = po.country_id  AND ad.postalcode = po.postalcode;

-- EOF
