-- Generel Ledger prefix: gl_ systemid: 5
-- Tables for the General Ledger Module
-- Copyright (c) 2001 The FreeMoney Project
-- This work is released under the GPL
-- $Id: create_tables_gl.sql,v 1.30 2001/10/28 18:29:24 kaare Exp $

-- Sequences
DROP SEQUENCE sy_ownersequence;
DROP SEQUENCE pd_vatsequence;
DROP SEQUENCE co_custgrpsequence;
DROP SEQUENCE pd_pgrpsequence;
DROP SEQUENCE gl_currencysequence;
DROP SEQUENCE gl_acsequence;
DROP SEQUENCE gl_coasequence;
DROP SEQUENCE gl_agsequence;
DROP SEQUENCE gl_batchsequence;
DROP SEQUENCE gl_journalsequence;
DROP SEQUENCE gl_reportsequence;
DROP SEQUENCE gl_reportrowsequence;
DROP SEQUENCE gl_reportcolsequence;

CREATE SEQUENCE sy_ownersequence INCREMENT 1;
CREATE SEQUENCE pd_vatsequence INCREMENT 1;
CREATE SEQUENCE co_custgrpsequence INCREMENT 1;
CREATE SEQUENCE pd_pgrpsequence INCREMENT 1;
CREATE SEQUENCE gl_currencysequence INCREMENT 1;
CREATE SEQUENCE gl_acsequence INCREMENT 1;
CREATE SEQUENCE gl_coasequence INCREMENT 1;
CREATE SEQUENCE gl_agsequence INCREMENT 1;
CREATE SEQUENCE gl_batchsequence INCREMENT 1;
CREATE SEQUENCE gl_journalsequence INCREMENT 1;
CREATE SEQUENCE gl_reportsequence INCREMENT 1;
CREATE SEQUENCE gl_reportrowsequence INCREMENT 1;
CREATE SEQUENCE gl_reportcolsequence INCREMENT 1;

--Tables
DROP TABLE sy_typetext;
DROP TABLE co_country;
DROP TABLE pd_vat;
DROP TABLE co_custgrp;
DROP TABLE pd_productgroup;
DROP TABLE gl_currency;
DROP TABLE gl_currencyrate;
DROP TABLE gl_acctname;
DROP TABLE gl_accounttype;
DROP TABLE gl_chartofaccount;
DROP TABLE gl_acctgrid;
DROP TABLE gl_balance;
DROP TABLE gl_batch;
DROP TABLE gl_journal;
DROP TABLE gl_reportrowhead;
DROP TABLE gl_reportrow;
DROP TABLE gl_reportcolhead;
DROP TABLE gl_reportcol;
DROP TABLE gl_report;

-- Several tables are put here because they are referenced
-- from Generel Ledger
CREATE TABLE sy_typetext (
       type                 integer NOT NULL,
       name                 varchar(128),
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (type)
);

CREATE TABLE co_country (
       country_id           integer NOT NULL DEFAULT nextval('co_countrysequence'),
       code                 char(20),
       name                 varchar(128),
       shipmodes            varchar(128),
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (country_id )
);

COMMENT ON TABLE co_country IS 'Country information';
COMMENT ON COLUMN co_country.country_id  IS 'Sequence number';
COMMENT ON COLUMN co_country.code IS 'Official code. E.g. DK for Denmark';
COMMENT ON COLUMN co_country.name IS 'Country name';
COMMENT ON COLUMN co_country.shipmodes IS 'Shipping modes - from Interchange. Maybe not needed?';
COMMENT ON COLUMN co_country.updated IS 'Timestamp for last update of this row';

-- VAT table from product module put here because it is referenced
-- from pd_productgroup which is referenced from Generel Ledger

CREATE TABLE pd_vat (
       vat_id               integer NOT NULL DEFAULT nextval('pd_vatsequence'),
       name                 varchar(128),
       percent              decimal,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (vat_id)
);

CREATE TABLE gl_currency (
       currency_id          integer NOT NULL DEFAULT nextval('gl_currencysequence'),
       code                 varchar(20) NOT NULL,
       country_id           integer NOT NULL,
       type                 integer NOT NULL,
       name                 varchar(128),
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (currency_id),
 FOREIGN KEY (country_id)   REFERENCES co_country,
 FOREIGN KEY (type)         REFERENCES sy_typetext
);

COMMENT ON TABLE gl_currency IS 'Currency information';
COMMENT ON COLUMN gl_currency.currency_id  IS 'Sequence number';
COMMENT ON COLUMN gl_currency.code IS 'Currency code';
COMMENT ON COLUMN gl_currency.country_id  IS 'Country reference';
COMMENT ON COLUMN gl_currency.type IS 'Type';
COMMENT ON COLUMN gl_currency.name IS 'Name';
COMMENT ON COLUMN gl_currency.updated IS 'Timestamp for last update of this row';

CREATE TABLE gl_currencyrate (
       currency_id          integer NOT NULL,
       rate                 decimal,
       startdate            timestamp,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (currency_id),
 FOREIGN KEY (currency_id)  REFERENCES gl_currency
);

COMMENT ON TABLE gl_currencyrate IS 'The currency rate towards the system rate';
COMMENT ON COLUMN gl_currencyrate.currency_id  IS 'Sequence number';
COMMENT ON COLUMN gl_currencyrate.rate IS 'Actual rate';
COMMENT ON COLUMN gl_currencyrate.startdate IS 'Starting date of this rate';
COMMENT ON COLUMN gl_currencyrate.updated IS 'Timestamp for last update of this row';

CREATE TABLE gl_acctname (
       ac_id                integer NOT NULL DEFAULT nextval('gl_acsequence'),
       name                 varchar(128),
       dimension            integer,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (ac_id)
);

COMMENT ON TABLE gl_acctname IS 'Names for entries in chartofaccount, productgroup and customer group';
COMMENT ON COLUMN gl_acctname.ac_id  IS 'Sequence number';
COMMENT ON COLUMN gl_acctname.name IS 'Name';
COMMENT ON COLUMN gl_acctname.dimension IS 'Dimension. 1 - Chart of account 2 - Product 3 - Customer';
COMMENT ON COLUMN gl_acctname.updated IS 'Timestamp for last update of this row';

-- Customergroup table from contact module put here because it is referenced
-- from Generel Ledger

CREATE TABLE co_custgrp (
       custgrp_id           integer NOT NULL DEFAULT nextval('co_custgrpsequence'),
       custgrp_nr           varchar(128),
       name                 varchar(128) NOT NULL,
       ac_id                integer,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (custgrp_id),
 FOREIGN KEY (ac_id)        REFERENCES gl_acctname
);

COMMENT ON TABLE co_custgrp IS 'Customer group. Defines third dimension in accounting.
Could also be used to define discount levels';
COMMENT ON COLUMN co_custgrp.custgrp_id  IS 'Sequence number';
COMMENT ON COLUMN co_custgrp.custgrp_nr IS 'Freely assignable number';
COMMENT ON COLUMN co_custgrp.name IS 'Name of customer group. E.g. "Dealers" or "Ordinary"';
COMMENT ON COLUMN co_custgrp.ac_id  IS 'Reference to accounting dimension';
COMMENT ON COLUMN co_custgrp.updated IS 'Timestamp for last update of this row';

-- Productgroup table from product module put here because it is referenced
-- from Generel Ledger

CREATE TABLE pd_productgroup (
       pgrp_id              integer NOT NULL DEFAULT nextval('pd_pgrpsequence'),
       vat_id               integer,
       name                 varchar(128),
       ac_id                integer,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (pgrp_id),
 FOREIGN KEY (vat_id)       REFERENCES pd_vat,
 FOREIGN KEY (ac_id)        REFERENCES gl_acctname
);

CREATE TABLE gl_accounttype (
       type                 integer NOT NULL,
       name                 varchar(128),
       status               boolean,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (type)
);

CREATE TABLE gl_chartofaccount (
       coa_id               integer NOT NULL DEFAULT nextval('gl_coasequence'),
       ac_id                integer NOT NULL,
       type                 integer,
       currency_id          integer,
       number               varchar(128),
       vat_id               integer,
       vat_coa              integer,
       soa                  boolean,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (coa_id),
 FOREIGN KEY (ac_id)        REFERENCES gl_acctname,
 FOREIGN KEY (type)         REFERENCES gl_accounttype,
 FOREIGN KEY (currency_id)  REFERENCES gl_currency,
 FOREIGN KEY (vat_id)       REFERENCES pd_vat,
 FOREIGN KEY (vat_coa)      REFERENCES gl_chartofaccount
);

COMMENT ON TABLE gl_chartofaccount IS 'Chart of account';
COMMENT ON COLUMN gl_chartofaccount.coa_id  IS 'Sequence number';
COMMENT ON COLUMN gl_chartofaccount.ac_id  IS 'Reference to acctname';
COMMENT ON COLUMN gl_chartofaccount.type IS 'Type. 1 - Assets 2 - Liabilities...';
COMMENT ON COLUMN gl_chartofaccount.currency_id  IS 'Currency';
COMMENT ON COLUMN gl_chartofaccount.number IS 'Public chart of account number';
COMMENT ON COLUMN gl_chartofaccount.updated IS 'Timestamp for last update of this row';

CREATE TABLE gl_acctgrid (
       ag_id                integer NOT NULL DEFAULT nextval('gl_agsequence'),
       dim1                 integer,
       dim2                 integer,
       dim3                 integer,
       dim4                 integer,
       dim5                 integer,
       type                 integer NOT NULL,
       currency_id          integer,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (ag_id),
 FOREIGN KEY (type)         REFERENCES sy_typetext,
 FOREIGN KEY (currency_id)  REFERENCES gl_currency,
 FOREIGN KEY (dim1)         REFERENCES gl_chartofaccount (coa_id),
 FOREIGN KEY (dim2)         REFERENCES pd_productgroup (pgrp_id),
 FOREIGN KEY (dim3)         REFERENCES co_custgrp (custgrp_id)
);

COMMENT ON TABLE gl_acctgrid IS 'Grid referenced by all dimensions. Currently 3: Account, product and customer';
COMMENT ON COLUMN gl_acctgrid.ag_id  IS 'Sequence number';
COMMENT ON COLUMN gl_acctgrid.dim1 IS 'First dimension - account';
COMMENT ON COLUMN gl_acctgrid.dim2 IS 'Second dimension - product';
COMMENT ON COLUMN gl_acctgrid.dim3 IS 'Third dimension - customer';
COMMENT ON COLUMN gl_acctgrid.dim4 IS 'Not yet used';
COMMENT ON COLUMN gl_acctgrid.dim5 IS 'Not yet used';
COMMENT ON COLUMN gl_acctgrid.type IS 'Type. 1 - Current 2 - budget 3 - best estimate 4 - last year ';
COMMENT ON COLUMN gl_acctgrid.currency_id  IS 'Currency';
COMMENT ON COLUMN gl_acctgrid.updated IS 'Timestamp for last update of this row';

CREATE TABLE gl_balance (
       ag_id                integer NOT NULL,
       periodnr             date NOT NULL,
       begindr              decimal DEFAULT 0,
       begincr              decimal DEFAULT 0,
       perioddr             decimal DEFAULT 0,
       periodcr             decimal DEFAULT 0,
       projecttodatedr      decimal DEFAULT 0,
       projecttodatecr      decimal DEFAULT 0,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (ag_id , periodnr),
 FOREIGN KEY (ag_id )       REFERENCES gl_acctgrid
);

COMMENT ON TABLE gl_balance IS 'Balance for period';
COMMENT ON COLUMN gl_balance.ag_id  IS 'Reference to gl_acctgrid';
COMMENT ON COLUMN gl_balance.periodnr IS 'Period number';
COMMENT ON COLUMN gl_balance.begindr IS 'Beginning balance for debit';
COMMENT ON COLUMN gl_balance.begincr IS 'Beginning balance for credit';
COMMENT ON COLUMN gl_balance.perioddr IS 'This month''s debit posting';
COMMENT ON COLUMN gl_balance.periodcr IS 'This month''s credit posting';
COMMENT ON COLUMN gl_balance.projecttodatedr IS 'I have no idea';
COMMENT ON COLUMN gl_balance.projecttodatecr IS 'I have no idea';
COMMENT ON COLUMN gl_balance.updated IS 'Timestamp for last update of this row';

CREATE TABLE gl_batch (
       batch_id             integer NOT NULL DEFAULT nextval('gl_batchsequence'),
       currency_id          integer,
       system_id            integer,
       type                 integer,
       batchnr              integer,
       name                 varchar(128),
       status               integer,
       totaldr              decimal,
       totalcr              decimal,
       createddate          timestamp,
       posteddate           timestamp,
       soa                  integer,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (batch_id),
 FOREIGN KEY (system_id)    REFERENCES sy_typetext,
 FOREIGN KEY (currency_id)  REFERENCES gl_currency,
 FOREIGN KEY (type)         REFERENCES sy_typetext,
 FOREIGN KEY (soa)          REFERENCES gl_chartofaccount
);

COMMENT ON TABLE gl_batch IS 'A batch is a logical way of grouping journal entries together.
A batch can be entered, but not accounted any time. A batch has to balance debit with credit before it can be accounted';
COMMENT ON COLUMN gl_batch.batch_id  IS 'Sequence number';
COMMENT ON COLUMN gl_batch.currency_id  IS 'Currency';
COMMENT ON COLUMN gl_batch.system_id  IS 'ID of originating system';
COMMENT ON COLUMN gl_batch.type IS 'Type. ??';
COMMENT ON COLUMN gl_batch.batchnr IS 'Used for - ?';
COMMENT ON COLUMN gl_batch.name IS 'Name';
COMMENT ON COLUMN gl_batch.status IS 'Status. 100 - entered 200 - accounted';
COMMENT ON COLUMN gl_batch.totaldr IS 'Total debit of all journals of this batch';
COMMENT ON COLUMN gl_batch.totalcr IS 'Total credit of all journals of this batch';
COMMENT ON COLUMN gl_batch.createddate IS 'Date of creation';
COMMENT ON COLUMN gl_batch.posteddate IS 'Date of posting / accounting';
COMMENT ON COLUMN gl_batch.updated IS 'Timestamp for last update of this row';

CREATE TABLE gl_journal (
       journal_id           integer NOT NULL DEFAULT nextval('gl_journalsequence'),
       batch_id             integer NOT NULL,
       journalnr            integer NOT NULL,
       ag_id                integer,
       entrydate            timestamp,
       entereddr            decimal,
       enteredcr            decimal,
       accounteddr          decimal,
       accountedcr          decimal,
       text                 varchar(128),
       accountingdate       timestamp,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (journal_id ),
 FOREIGN KEY (batch_id )    REFERENCES gl_batch
  ON DELETE CASCADE
  ON UPDATE CASCADE,
 FOREIGN KEY (ag_id )       REFERENCES gl_acctgrid
);

COMMENT ON TABLE gl_journal IS 'A journal entry is the lowest atomic element of accounting';
COMMENT ON COLUMN gl_journal.journal_id  IS 'Sequence number';
COMMENT ON COLUMN gl_journal.batch_id  IS 'Reference to gl_batch';
COMMENT ON COLUMN gl_journal.journalnr IS 'Journal number';
COMMENT ON COLUMN gl_journal.ag_id  IS 'Refence to acctgrid';
COMMENT ON COLUMN gl_journal.entrydate IS 'Date of entry';
COMMENT ON COLUMN gl_journal.entereddr IS 'Entered debit amount';
COMMENT ON COLUMN gl_journal.enteredcr IS 'Entered credit amount';
COMMENT ON COLUMN gl_journal.accounteddr IS 'Accounted debit amount. (What for?)';
COMMENT ON COLUMN gl_journal.accountedcr IS 'Accounted credit amount. (What for?)';
COMMENT ON COLUMN gl_journal.text IS 'Text';
COMMENT ON COLUMN gl_journal.accountingdate IS 'Time for accounting';
COMMENT ON COLUMN gl_journal.updated IS 'Timestamp for last update of this row';

CREATE TABLE gl_reportrowhead (
       row_id               integer NOT NULL DEFAULT nextval('gl_reportrowsequence'),
       description          varchar(512),
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (row_id)
);

CREATE TABLE gl_reportrow (
       row_id               integer NOT NULL,
       linenr               integer,
       rowtype              integer,
       label                varchar(128),
       rowtext              varchar(512),
       acctinfo1            varchar(128),
       acctinfo2            varchar(128),
       sign1                integer,
       fromline             varchar(128),
       sign2                integer,
       toline               varchar(128),
       aggregate            boolean,
       hide                 boolean,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (row_id, linenr),
 FOREIGN KEY (row_id)       REFERENCES gl_reportrowhead
 ON UPDATE CASCADE
 ON DELETE CASCADE
);

CREATE TABLE gl_reportcolhead (
       col_id               integer NOT NULL DEFAULT nextval('gl_reportcolsequence'),
       description          varchar(512),
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (col_id)
);

CREATE TABLE gl_reportcol (
       col_id               integer NOT NULL,
       linenr               integer,
       coltext              varchar(512),
       amounttype           integer,
       coltype              integer,
       sign1                char,
       fromperiod           integer,
       sign2                char,
       toperiod             integer,
       relative             boolean,
       aggregate            boolean,
       fsign1               integer,
       fromline             integer,
       fsign2               integer,
       toline               integer,
       hide                 boolean,
       rounding             integer,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (col_id, linenr),
 FOREIGN KEY (amounttype)   REFERENCES sy_typetext (type),
 FOREIGN KEY (col_id)       REFERENCES gl_reportcolhead
 ON UPDATE CASCADE
 ON DELETE CASCADE
);

CREATE TABLE gl_report (
       glreport_id          integer NOT NULL DEFAULT nextval('gl_reportsequence'),
       reportnr             integer,
       name                 varchar(128),
       description          varchar(128),
       row_id               integer,
       col_id               integer,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (glreport_id),
 FOREIGN KEY (row_id)       REFERENCES gl_reportrowhead,
 FOREIGN KEY (col_id)       REFERENCES gl_reportcolhead
);

-- Views

-- Functions

-- Triggers
